@@ -4,11 +4,11 @@
 from __future__ import annotations
 
 import bdb
+from collections.abc import Callable
 import dataclasses
 import os
 import sys
 import types
-from typing import Callable
 from typing import cast
 from typing import final
 from typing import Generic
@@ -16,6 +16,7 @@ from typing import Literal
 from typing import TYPE_CHECKING
 from typing import TypeVar
 
+from .config import Config
 from .reports import BaseReport
 from .reports import CollectErrorRepr
 from .reports import CollectReport
@@ -61,19 +62,21 @@ def pytest_addoption(parser: Parser) -> None:
         "--durations-min",
         action="store",
         type=float,
-        default=0.005,
+        default=None,
         metavar="N",
         help="Minimal duration in seconds for inclusion in slowest list. "
-        "Default: 0.005.",
+        "Default: 0.005 (or 0.0 if -vv is given).",
     )
 
 
 def pytest_terminal_summary(terminalreporter: TerminalReporter) -> None:
     durations = terminalreporter.config.option.durations
     durations_min = terminalreporter.config.option.durations_min
-    verbose = terminalreporter.config.getvalue("verbose")
+    verbose = terminalreporter.config.get_verbosity()
     if durations is None:
         return
+    if durations_min is None:
+        durations_min = 0.005 if verbose < 2 else 0.0
     tr = terminalreporter
     dlist = []
     for replist in tr.stats.values():
@@ -90,11 +93,13 @@ def pytest_terminal_summary(terminalreporter: TerminalReporter) -> None:
         dlist = dlist[:durations]
 
     for i, rep in enumerate(dlist):
-        if verbose < 2 and rep.duration < durations_min:
+        if rep.duration < durations_min:
             tr.write_line("")
-            tr.write_line(
-                f"({len(dlist) - i} durations < {durations_min:g}s hidden.  Use -vv to show these durations.)"
-            )
+            message = f"({len(dlist) - i} durations < {durations_min:g}s hidden."
+            if terminalreporter.config.option.durations_min is None:
+                message += "  Use -vv to show these durations."
+            message += ")"
+            tr.write_line(message)
             break
         tr.write_line(f"{rep.duration:02.2f}s {rep.when:<8} {rep.nodeid}")
 
@@ -167,7 +172,7 @@ def pytest_runtest_call(item: Item) -> None:
         del sys.last_value
         del sys.last_traceback
         if sys.version_info >= (3, 12, 0):
-            del sys.last_exc
+            del sys.last_exc  # type:ignore[attr-defined]
     except AttributeError:
         pass
     try:
@@ -177,7 +182,7 @@ def pytest_runtest_call(item: Item) -> None:
         sys.last_type = type(e)
         sys.last_value = e
         if sys.version_info >= (3, 12, 0):
-            sys.last_exc = e
+            sys.last_exc = e  # type:ignore[attr-defined]
         assert e.__traceback__ is not None
         # Skip *this* frame
         sys.last_traceback = e.__traceback__.tb_next
@@ -235,11 +240,11 @@ def call_and_report(
         runtest_hook = ihook.pytest_runtest_teardown
     else:
         assert False, f"Unhandled runtest hook case: {when}"
-    reraise: tuple[type[BaseException], ...] = (Exit,)
-    if not item.config.getoption("usepdb", False):
-        reraise += (KeyboardInterrupt,)
+
     call = CallInfo.from_call(
-        lambda: runtest_hook(item=item, **kwds), when=when, reraise=reraise
+        lambda: runtest_hook(item=item, **kwds),
+        when=when,
+        reraise=get_reraise_exceptions(item.config),
     )
     report: TestReport = ihook.pytest_runtest_makereport(item=item, call=call)
     if log:
@@ -249,6 +254,14 @@ def call_and_report(
     return report
 
 
+def get_reraise_exceptions(config: Config) -> tuple[type[BaseException], ...]:
+    """Return exception types that should not be suppressed in general."""
+    reraise: tuple[type[BaseException], ...] = (Exit,)
+    if not config.getoption("usepdb", False):
+        reraise += (KeyboardInterrupt,)
+    return reraise
+
+
 def check_interactive_exception(call: CallInfo[object], report: BaseReport) -> bool:
     """Check whether the call raised an exception that should be reported as
     interactive."""
@@ -258,7 +271,7 @@ def check_interactive_exception(call: CallInfo[object], report: BaseReport) -> b
     if hasattr(report, "wasxfail"):
         # Exception was expected.
         return False
-    if isinstance(call.excinfo.value, (Skipped, bdb.BdbQuit)):
+    if isinstance(call.excinfo.value, Skipped | bdb.BdbQuit):
         # Special control flow exception.
         return False
     return True
@@ -335,8 +348,7 @@ class CallInfo(Generic[TResult]):
             function, instead of being wrapped in the CallInfo.
         """
         excinfo = None
-        start = timing.time()
-        precise_start = timing.perf_counter()
+        instant = timing.Instant()
         try:
             result: TResult | None = func()
         except BaseException:
@@ -344,14 +356,11 @@ class CallInfo(Generic[TResult]):
             if reraise is not None and isinstance(excinfo.value, reraise):
                 raise
             result = None
-        # use the perf counter
-        precise_stop = timing.perf_counter()
-        duration = precise_stop - precise_start
-        stop = timing.time()
+        duration = instant.elapsed()
         return cls(
-            start=start,
-            stop=stop,
-            duration=duration,
+            start=duration.start.time,
+            stop=duration.stop.time,
+            duration=duration.seconds,
             when=when,
             result=result,
             excinfo=excinfo,
@@ -533,7 +542,7 @@ class SetupState:
         When nextitem is None (meaning we're at the last item), the entire
         stack is torn down.
         """
-        needed_collectors = nextitem and nextitem.listchain() or []
+        needed_collectors = (nextitem and nextitem.listchain()) or []
         exceptions: list[BaseException] = []
         while self.stack:
             if list(self.stack.keys()) == needed_collectors[: len(self.stack)]:
